CONTROL_KEYS = {"Control_L", "Control_R"}
PAST_WINDOW_SIZE = (400, 600)
SWITCHER_WINDOW_SIZE = (400, 600)
TOC_WINDOW_SIZE = (400, 550)
INFO_WINDOW_SIZE = (300, 300)
SEARCH_WINDOW_SIZE = (400, 650)
WINDOW_MINSIZE = (350, 350)
